# $Id: LSCsegFindSegGenerator.py,v 1.13 2006/02/23 01:11:59 kipp Exp $
#
# Copyright (C) 2005  Kipp C. Cannon
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#
# Preamble.
#

import logging

from glue import lal
from glue import segments
import onasys.SegGenerator
from onasys import onasysd


#
# A single-instrument segment generator based on queries to an LSCsegFind
# server with optional confirmation queries to an LSCdataFind server.
#

class SegGeneratorConfig(onasys.SegGenerator.SegGeneratorConfig, onasysd.SegFindConfig):
	def defaults(self):
		return onasysd.SegFindConfig.defaults(self) + [
			("datafind", "False")
		]

	def parse(self):
		onasysd.SegFindConfig.parse(self)
		if self.parser.getboolean(self.section, "datafind"):
			self.datafindconfig = onasysd.DataFindConfig(self.filename)
			self.datafindconfig.parse()
			self.datafind = onasysd.DataFind(self.datafindconfig)
		else:
			self.datafind = None
		self.initial_epoch = lal.LIGOTimeGPS(self.parser.get(self.section, "initial_epoch"))

	def log(self):
		l = onasys.SegGenerator.SegGeneratorConfig.log(self)
		if self.datafind:
			l += self.datafindconfig.log()
		return l

	def validate(self):
		if self.datafind:
			self.datafindconfig.validate()


class SegGenerator(onasys.SegGenerator.SegGenerator):
	def __init__(self, config):
		self.config = config
		self.segfind = onasysd.SegFind(config)
		self.search_window = onasysd.SearchWindow()

	def get_state(self):
		return self.search_window

	def set_state(self, state):
		self.search_window = state

	def generate(self, available, clock):
		# Get old "available" segment list.
		old_avail = available.get("available", segments.segmentlist())

		# Determine the window in which to search for segments.
		self.search_window.next(self.config.initial_epoch, clock.maxend(), self.config.min_query_length)
		window = self.search_window.window

		# Always at least query from the end of the old available
		# segment list.
		if len(old_avail):
			if old_avail[-1][1] < window[0]:
				window |= segments.segment(old_avail[-1][1], window[1])

		# Put a useful message in the log.
		logging.info(self.segfind.command(window))

		# Do an LSCsegFind query.
		try:
			list = self.segfind.list(window)
		except Exception, e:
			raise onasys.SegGenerator.SegGeneratorError, "LSCsegFind query failure: " + str(e)
		logging.info("LSCsegFind response = " + str(list))

		# If needed, compute the intersection of the segments and
		# the response of an LSCdataFind query.
		if self.config.datafind:
			try:
				datafindlist = segments.segmentlist()
				# query only for those segments we haven't
				# seen before
				for seg in list - old_avail:
					logging.info(self.config.datafind.command(seg))
					datafindlist |= self.config.datafind.list(seg)
				logging.info("LSCdataFind response = " + str(datafindlist))
				list &= datafindlist
			except Exception, e:
				raise onasys.SegGenerator.SegGeneratorError, "LSCdataFind query failure: " + str(e)

		return {"available": list}
